#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################



import abc
from typing import Optional, List, Tuple
import time
import datetime as dt
import numpy as np
from docx import Document
from docx.shared import Inches, Cm
from docx.enum.style import WD_STYLE_TYPE
from docx.enum.table import WD_TABLE_DIRECTION
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
import nardascripting.base.toolbox as tb
from .basedata import DocItem, IDocDataItem
from . import langhelper as lh
_ = lh.CURRENT_GETTEXT


class DocBaseDataItem(IDocDataItem):
    """Class that represents measurement basic information"""

    def __init__(self, screenshot_base_path: Optional[Path] = None):
        """Class initialization"""
        super().__init__(screenshot_base_path)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return "MEAS_BASE_DATA"

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("General Measurement Details")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Overview")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 0

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            self._meas_settings.clear()

            gnss = signalshark.scpi.sense.get_gnss_data()
            meas_pos_str = ""
            if gnss is not None:
                meas_pos_str = "({}, {})".format(gnss.latitude, gnss.longitude)
            self._meas_settings.append(DocItem(_("GNSS Position"), meas_pos_str))

            rf_input = signalshark.scpi.sense.get_input()
            self._meas_settings.append(DocItem(_("Input"), rf_input))

            unit = signalshark.scpi.display.get_unit()
            self._meas_settings.append(DocItem(_("Unit"), unit))

            att = signalshark.scpi.sense.get_attenuator()
            self._meas_settings.append(DocItem(_("Atten."), att, "dB"))

            ref_level = signalshark.scpi.sense.get_reference_level()
            self._meas_settings.append(DocItem(_("Ref. Lev."), ref_level, unit))

            ref_level_offset_enabled = signalshark.scpi.sense.get_reference_level_offset_enable()
            if ref_level_offset_enabled:
                ref_level_offset = signalshark.scpi.sense.get_reference_level_offset()
                self._meas_settings.append(DocItem(_("Ref. Level Offset"), ref_level_offset, "dB"))

            signalshark.scpi.view.size_restore()
            time.sleep(1.5)
            return self._take_screenshot(signalshark)
        except:
            return False


class DocSpectrumBaseDataItem(IDocDataItem):
    """Base class for spectrum data"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            self._meas_settings.clear()

            freq_entry_mode = signalshark.scpi.spectrum.get_frequency_entry_mode()
            if freq_entry_mode == FreqEntryModes.FCENT_FSPAN:
                fcent = signalshark.scpi.spectrum.get_frequency_center()
                self._meas_settings.append(DocItem(_("Fcent"), fcent, "Hz", True))
                fspan = signalshark.scpi.spectrum.get_frequency_span()
                self._meas_settings.append(DocItem(_("Fspan"), fspan, "Hz", True))
            else:
                fstart = signalshark.scpi.spectrum.get_frequency_start()
                self._meas_settings.append(DocItem(_("Fstart"), fstart, "Hz", True))
                fstop = signalshark.scpi.spectrum.get_frequency_stop()
                self._meas_settings.append(DocItem(_("Fstop"), fstop, "Hz", True))

            rbw = signalshark.scpi.spectrum.get_rbw()
            self._meas_settings.append(DocItem(_("RBW"), rbw, "Hz", True))

            meastime = signalshark.scpi.spectrum.get_measurement_time()
            self._meas_settings.append(DocItem(_("Meas. Time"), meastime, "s", True))

            scan_no = signalshark.scpi.spectrum.get_scan_number()
            self._meas_settings.append(DocItem(_("Scan No."), scan_no))

            scan_count = signalshark.scpi.spectrum.get_scan_count()
            self._meas_settings.append(DocItem(_("Scan Count"), scan_count))

            return True
        except:
            return False


class DocSpectrumDataItem(DocSpectrumBaseDataItem):
    """Class that represents a Spectrum View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.SPECTRUM)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Spectrum View")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Spectrum_View")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 1

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            check = super().read_data_from_device(signalshark)
            return self._take_screenshot(signalshark) & check
        except:
            return False


class DocSpectrogramDataItem(DocSpectrumBaseDataItem):
    """Class that represents a Spectrogram View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.SPECTROGRAM)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Spectrogram View")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Spectrogram_View")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 2

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            check = super().read_data_from_device(signalshark)

            sgram_frames = signalshark.scpi.sgram.get_frame_count()
            self._meas_settings.append(DocItem(_("Sgram Frames"), sgram_frames))

            return self._take_screenshot(signalshark) & check
        except:
            return False


class DocMarkerDataItem(IDocDataItem):
    """Class that represents marker data"""

    def __init__(self, screenshot_base_path: Optional[Path] = None):
        """Class initialization"""
        super().__init__(screenshot_base_path)

        # Marker Table Header & Data Init
        self._mtbl_header = []
        self._mtbl_data = []

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return "MARKER_DATA"

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Marker")

    @property
    def meas_result_headline(self) -> str:
        """Gets the measurement result headline for this data item. """
        return _("Marker Table Values")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 3

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            # Read settings:
            # -------------------------------------------------------------------------------
            self._meas_settings.clear()
            unit = signalshark.scpi.display.get_unit()

            auto_peak_enable = signalshark.scpi.marker.get_spectrum_search_auto_peak_enable()
            auto_peak_enable_onoff = _("On") if auto_peak_enable else _("Off")
            self._meas_settings.append(DocItem(_("Auto Peak Search for M1"), auto_peak_enable_onoff))

            track_peaks_enable = signalshark.scpi.marker.get_spectrum_search_track_peaks_enable()
            track_peaks_enable_onoff = _("On") if track_peaks_enable else _("Off")
            self._meas_settings.append(DocItem(_("Exclude LO"), track_peaks_enable_onoff))

            sgram_range = signalshark.scpi.marker.get_spectrum_search_sgram_range()
            self._meas_settings.append(DocItem(_("Spectrogram Search range"), sgram_range))

            limits_enable = signalshark.scpi.marker.get_spectrum_search_limits_enable()
            if limits_enable:
                low_freq_lim = signalshark.scpi.marker.get_spectrum_search_frequency_lower()
                self._meas_settings.append(DocItem(_("Lower Freq. Lim."), low_freq_lim, "Hz", True))

                upper_freq_lim = signalshark.scpi.marker.get_spectrum_search_frequency_upper()
                self._meas_settings.append(DocItem(_("Upper Freq. Lim."), upper_freq_lim, "Hz", True))

            threshold = signalshark.scpi.marker.get_spectrum_search_threshold()
            self._meas_settings.append(DocItem(_("Threshold"), threshold, unit, True))

            peak_excursion_enable = signalshark.scpi.marker.get_spectrum_search_peak_excursion_enable()
            if peak_excursion_enable:
                peak_excursion = signalshark.scpi.marker.get_spectrum_search_peak_excursion()
                self._meas_settings.append(DocItem(_("Peak Excursion"), peak_excursion, "dB"))

            exclude_lo = signalshark.scpi.marker.get_spectrum_search_loexclude_enable()
            exclude_lo_onoff = _("On") if exclude_lo else _("Off")
            self._meas_settings.append(DocItem(_("Exclude LO"), exclude_lo_onoff))

            nbw_auto = signalshark.scpi.marker.get_spectrum_function_noise_nbw_auto()
            nbw_auto_onoff = _("On") if nbw_auto else _("Off")
            self._meas_settings.append(DocItem(_("Noise Marker Auto NBW"), nbw_auto_onoff))

            noise_nbw = signalshark.scpi.marker.get_spectrum_function_noise_nbw()
            self._meas_settings.append(DocItem(_("Noise Marker NBW"), noise_nbw, "Hz", True))

            cpower_cbw = signalshark.scpi.marker.get_spectrum_function_cpower_cbw()
            self._meas_settings.append(DocItem(_("Chanel Power CBW"), cpower_cbw, "Hz", True))

            ocbw_mode = signalshark.scpi.marker.get_spectrum_function_ocbw_mode()
            self._meas_settings.append(DocItem(_("Occupied Bandwidth Mode"), ocbw_mode))

            if ocbw_mode == OcbwModes.PERCENT:
                ocbw_percent = signalshark.scpi.marker.get_spectrum_function_ocbw_percent()
                self._meas_settings.append(DocItem(_("Occupied Bandwidth Percent"), ocbw_percent, "%"))

            else:
                ocbw_xdb = signalshark.scpi.marker.get_spectrum_function_ocbw_xdb()
                self._meas_settings.append(DocItem(_("Occupied Bandwidth x dB"), ocbw_xdb, "dB"))

                ocbw_ntrials = signalshark.scpi.marker.get_spectrum_function_ocbw_ntrials()
                self._meas_settings.append(DocItem(_("Occupied Bandwidth Ntrials"), ocbw_ntrials))

            ocbw_threshold = signalshark.scpi.marker.get_spectrum_function_ocbw_threshold()
            self._meas_settings.append(DocItem(_("Occupied Bandwidth Threshold"), ocbw_threshold, unit, True))

            # Read measurement results:
            # -------------------------------------------------------------------------------
            enabled_markers = signalshark.scpi.marker.get_spectrum_marker_list()
            if len(enabled_markers) < 1:
                return True

            # Peak Table Header & Data Init
            self._mtbl_header = [_("Marker"), _("Ref."), _("Trace"), _("Frequency"), _("Time"), _("Trace Value"),
                                 _("Function"), _("Function Value"), _("Function Value 2"), _("Function Value 3")]
            self._mtbl_data = []

            unit = signalshark.scpi.display.get_unit()
            for marker, index in enabled_markers:
                m_ref = str(signalshark.scpi.marker.get_spectrum_reference(index))
                m_trace = str(signalshark.scpi.marker.get_spectrum_trace(index))
                m_freq = tb.value_to_unit_string(signalshark.scpi.marker.get_spectrum_data_frequency(index), "Hz", True)
                m_time = tb.value_to_unit_string(signalshark.scpi.marker.get_spectrum_data_time(index), "s", True)
                m_trace_val = tb.value_to_unit_string(signalshark.scpi.marker.get_spectrum_data_level(index), unit)

                func = signalshark.scpi.marker.get_spectrum_function(index)
                m_funk = str(func)
                if func == SpecMarkerFunctions.CHANNEL_POWER:
                    m_func_val = tb.value_to_unit_string(signalshark.scpi.marker.get_spectrum_function_cpower_data(index), unit)
                    m_func_val2 = ""
                    m_func_val3 = ""
                elif func == SpecMarkerFunctions.NOISE:
                    m_func_val = tb.value_to_unit_string(signalshark.scpi.marker.get_spectrum_function_noise_data(index),
                                                 unit + "/Hz^0.5", True)
                    m_func_val2 = ""
                    m_func_val3 = ""
                elif func == SpecMarkerFunctions.OCCUPIED_BANDWIDTH:
                    # OCBW, Fcent, Channel Power
                    func_val, func_val2, func_val3 = signalshark.scpi.marker.get_spectrum_function_ocbw_data(index)
                    m_func_val = tb.value_to_unit_string(func_val, "Hz", True)
                    m_func_val2 = tb.value_to_unit_string(func_val2, "Hz", True)
                    m_func_val3 = tb.value_to_unit_string(func_val2, unit)

                elif func == SpecMarkerFunctions.TRANSMITTER:
                    # Transmitter name, Transmitter type, Transmitter comment
                    m_func_val, func_val2, m_func_val3 = signalshark.scpi.marker.get_spectrum_function_transmitter_data(
                        index)
                    m_func_val2 = str(func_val2)
                else:
                    m_func_val = ""
                    m_func_val2 = ""
                    m_func_val3 = ""

                self._mtbl_data.append([marker, m_ref, m_trace, m_freq, m_time, m_trace_val,
                                        m_funk, m_func_val, m_func_val2, m_func_val3])

            return True
        except:
            return False

    def render_data_to_doc(self, document: Document) -> bool:
        """Renders measurement settings and data of this data item into the given document.

        :param document: The Word document.
        :return: Returns True if succeeded.
        """
        try:
            # Get styles
            stl_meas_result = self._get_doc_style(document, "MeasResult", WD_STYLE_TYPE.PARAGRAPH)
            stl_result_tbl = self._get_doc_style(document, "ResultTbl", WD_STYLE_TYPE.TABLE)

            # Render settings:
            # -------------------------------------------------------------------------------
            settings_result = super().render_data_to_doc(document)

            # Render measurement results:
            # -------------------------------------------------------------------------------
            document.add_heading(self.meas_result_headline, 2)
            tbl_rows = len(self._mtbl_data)
            tbl_cols = len(self._mtbl_header)

            if tbl_rows < 1:
                return True

            peak_table = document.add_table(rows=tbl_rows + 1, cols=tbl_cols)
            peak_table.style = stl_result_tbl
            peak_table.table_direction = WD_TABLE_DIRECTION.LTR
            peak_table.autofit = True

            for col in range(tbl_cols):
                peak_table.cell(0, col).text = self._mtbl_header[col]

            for row in range(tbl_rows):
                for col in range(tbl_cols):
                    peak_table.cell(row + 1, col).text = self._mtbl_data[row][col]

            return settings_result
        except:
            return False


class DocPeakTblDataItem(DocSpectrumBaseDataItem):
    """Class that represents a Peak Table View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

        self._ptbl_data_cnt = 0

        # Peak Table Header & Data Init
        self._ptbl_header = []
        self._ptbl_data = []

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.PEAK_TABLE)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Peak Table")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Peak_Table_View")

    @property
    def meas_result_headline(self) -> str:
        """Gets the measurement result headline for this data item. """
        return _("Peak Table Data")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 4

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            # Read settings:
            # -------------------------------------------------------------------------------
            check = super().read_data_from_device(signalshark)
            unit = signalshark.scpi.display.get_unit()

            limits_enable = signalshark.scpi.marker.get_spectrum_search_limits_enable()
            if limits_enable:
                low_freq_lim = signalshark.scpi.marker.get_spectrum_search_frequency_lower()
                self._meas_settings.append(DocItem(_("Lower Freq. Lim."), low_freq_lim, "Hz", True))

                upper_freq_lim = signalshark.scpi.marker.get_spectrum_search_frequency_upper()
                self._meas_settings.append(DocItem(_("Upper Freq. Lim."), upper_freq_lim, "Hz", True))

            threshold = signalshark.scpi.marker.get_spectrum_search_threshold()
            self._meas_settings.append(DocItem(_("Threshold"), threshold, unit, True))

            peak_excursion_enable = signalshark.scpi.marker.get_spectrum_search_peak_excursion_enable()
            if peak_excursion_enable:
                peak_excursion = signalshark.scpi.marker.get_spectrum_search_peak_excursion()
                self._meas_settings.append(DocItem(_("Peak Excursion"), peak_excursion, "dB"))

            exclude_lo = signalshark.scpi.marker.get_spectrum_search_loexclude_enable()
            exclude_lo_onoff = _("On") if exclude_lo else _("Off")
            self._meas_settings.append(DocItem(_("Exclude LO"), exclude_lo_onoff))

            # Read measurement results:
            # -------------------------------------------------------------------------------
            self._ptbl_data_cnt = signalshark.scpi.peaktable.get_data_count()

            # Peak Table Header & Data Init
            self._ptbl_header = [_("No."), _("Frequency")]
            self._ptbl_data = []

            # Get frequency information
            frequencies = []
            freq_data = signalshark.scpi.peaktable.get_data_frequency()
            no_list = []
            no = 1
            for item in freq_data:
                no_list.append(str(no))
                no += 1
                frequencies.append(tb.value_to_unit_string(item, "Hz", True))
            self._ptbl_data.append(no_list)
            self._ptbl_data.append(frequencies)

            # Get trace data
            traces = signalshark.scpi.spectrum.get_trace_list()
            for trace in traces:
                self._ptbl_header.append(str(trace))
                trace_values = []
                trace_data = signalshark.scpi.peaktable.get_data_level(trace)
                for item in trace_data:
                    trace_values.append(tb.value_to_unit_string(item, unit, True))
                self._ptbl_data.append(trace_values)

            return self._take_screenshot(signalshark) & check
        except:
            return False

    def render_data_to_doc(self, document: Document) -> bool:
        """Renders measurement settings and data of this data item into the given document.

        :param document: The Word document.
        :return: Returns True if succeeded.
        """
        try:
            # Get styles
            stl_meas_result = self._get_doc_style(document, "MeasResult", WD_STYLE_TYPE.PARAGRAPH)
            stl_result_tbl = self._get_doc_style(document, "ResultTbl", WD_STYLE_TYPE.TABLE)

            # Render settings:
            # -------------------------------------------------------------------------------
            settings_result = super().render_data_to_doc(document)

            # Render measurement results:
            # -------------------------------------------------------------------------------
            document.add_page_break()
            document.add_heading(self.meas_result_headline, 2)

            tbl_rows = self._ptbl_data_cnt
            tbl_cols = len(self._ptbl_header)

            peak_table = document.add_table(rows=tbl_rows + 1, cols=tbl_cols)
            peak_table.style = stl_result_tbl
            peak_table.table_direction = WD_TABLE_DIRECTION.LTR
            peak_table.autofit = True

            for col in range(tbl_cols):
                peak_table.cell(0, col).text = self._ptbl_header[col]

            for row in range(tbl_rows):
                for col in range(tbl_cols):
                    col_data = self._ptbl_data[col]
                    peak_table.cell(row + 1, col).text = col_data[row]

            return settings_result
        except Exception as e:
            return False


class DocPersistenceDataItem(DocSpectrumBaseDataItem):
    """Class that represents a Persistence View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.PERSISTENCE)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Persistence View")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Persistence_View")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 5

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            check = super().read_data_from_device(signalshark)
            return self._take_screenshot(signalshark) & check
        except:
            return False


class DocLevelMeterDataItem(IDocDataItem):
    """Class that represents a Level Meter View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.LEVEL)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Level Meter View")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Level_Meter_View")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 6

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            # Read settings:
            # -------------------------------------------------------------------------------
            self._meas_settings.clear()

            unit = signalshark.scpi.display.get_unit()

            ftune = signalshark.scpi.levelmeter.get_frequency_tune()
            self._meas_settings.append(DocItem(_("Ftune"), ftune, "Hz", True))

            cbw = signalshark.scpi.levelmeter.get_cbw()
            self._meas_settings.append(DocItem(_("CBW"), cbw, "Hz", True))

            cbw_filter_type = signalshark.scpi.levelmeter.get_cbw_filter_type()
            self._meas_settings.append(DocItem(_("CBW Filter Type"), cbw_filter_type))

            preavg = signalshark.scpi.levelmeter.get_pre_avg()
            if preavg > 0:
                self._meas_settings.append(DocItem(_("Pre Avg."), preavg, "s", True))

            postavg = signalshark.scpi.levelmeter.get_post_avg()
            if postavg > 0:
                self._meas_settings.append(DocItem(_("Post Avg."), postavg, "s", True))

            meastime = signalshark.scpi.levelmeter.get_measurement_time()
            self._meas_settings.append(DocItem(_("Meas. Time"), meastime, "s", True))

            det_infinite = signalshark.scpi.levelmeter.get_detector_infinite()
            det_infinite_onoff = _("On") if det_infinite else _("Off")
            self._meas_settings.append(DocItem(_("Detector infinite min max"), det_infinite_onoff))

            scan_no = signalshark.scpi.levelmeter.get_scan_number()
            self._meas_settings.append(DocItem(_("Scan No."), scan_no))

            scan_count = signalshark.scpi.levelmeter.get_scan_count()
            self._meas_settings.append(DocItem(_("Scan Count"), scan_count))

            # Read measurement results:
            # -------------------------------------------------------------------------------
            # Get detector 1 to 3:
            for i in range(1, 4):
                det_type = signalshark.scpi.levelmeter.get_detector(i)
                if det_type != LevDetectorModes.OFF:
                    minmax_suffix = "Max"
                    if det_type == LevDetectorModes.MPk:
                        minmax_suffix = "Min"
                    det_name = _("Detector {} ({})".format(i, det_type))
                    det_minmaxname = _("Detector {} ({} {})".format(i, det_type, minmax_suffix))
                    det_val, det_minmax_val = signalshark.scpi.levelmeter.get_data_detector(i)
                    self._meas_results.append(DocItem(det_name, det_val, unit, True))
                    self._meas_results.append(DocItem(det_minmaxname, det_minmax_val, unit, True))


            # Get Modulation detector:
            mod_det_type = signalshark.scpi.levelmeter.get_detector_modulation()
            if mod_det_type != DetectorModConfigs.OFF:
                self._meas_results.append(DocItem(_("Modulation Type"), mod_det_type))
                mod_det_values = signalshark.scpi.levelmeter.get_data_detector_modulation()

                for i, item in enumerate(mod_det_values):
                    mod_det = mod_det_values[i].mod_det
                    unit_str = signalshark.scpi.levelmeter.get_detector_modulation_Unit_string(mod_det)
                    add_prefix = unit_str == "Hz"
                    det_value = mod_det_values[i].det_value
                    det_trace_mvalue = mod_det_values[i].det_trace_mvalue
                    self._meas_results.append(DocItem(_("Modulation Detector {}").format(i+1), mod_det))
                    self._meas_results.append(DocItem(_("Modulation Detector {} Value").format(i+1),
                                                      det_value, unit_str, add_prefix))
                    self._meas_results.append(DocItem(_("Modulation Detector {} Max Value").format(i+1),
                                                      det_trace_mvalue, unit_str, add_prefix))

            return self._take_screenshot(signalshark)
        except:
            return False


class DocHScanDataItem(IDocDataItem):
    """Class that represents a Horizontal Scan View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.HORIZONTAL_SCAN)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Horizontal Scan View")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Horizontal_Scan_View")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 7

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            self._meas_settings.clear()

            scan_type = signalshark.scpi.horizontal.get_scan_type()
            self._meas_settings.append(DocItem(_("Scan Type"), scan_type))

            detector = signalshark.scpi.horizontal.get_scan_detector()
            self._meas_settings.append(DocItem(_("Detector"), detector))

            ftune = signalshark.scpi.levelmeter.get_frequency_tune()
            self._meas_settings.append(DocItem(_("Ftune"), ftune, "Hz", True))

            cbw = signalshark.scpi.levelmeter.get_cbw()
            self._meas_settings.append(DocItem(_("CBW"), cbw, "Hz", True))

            preavg = signalshark.scpi.levelmeter.get_pre_avg()
            if preavg > 0:
                self._meas_settings.append(DocItem(_("Pre Avg."), preavg, "s", True))

            postavg = signalshark.scpi.levelmeter.get_post_avg()
            if postavg > 0:
                self._meas_settings.append(DocItem(_("Post Avg."), postavg, "s", True))

            meastime = signalshark.scpi.levelmeter.get_measurement_time()
            self._meas_settings.append(DocItem(_("Meas. Time"), meastime, "s", True))

            scan_no = signalshark.scpi.levelmeter.get_scan_number()
            self._meas_settings.append(DocItem(_("Scan No."), scan_no))

            scan_count = signalshark.scpi.levelmeter.get_scan_count()
            self._meas_settings.append(DocItem(_("Scan Count"), scan_count))

            return self._take_screenshot(signalshark)
        except:
            return False


class DocBearingDataItem(IDocDataItem):
    """Class that represents a Bearing View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.BEARING)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Bearing View")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Bearing_View")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 8

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            self._meas_settings.clear()

            unit = signalshark.scpi.display.get_unit()

            north_ref = signalshark.scpi.bearing.get_north_reference()
            self._meas_settings.append(DocItem(_("North Reference"), north_ref))

            if north_ref == BearingNorthRefs.COMPASS:
                magn_decl = signalshark.scpi.__ll_settings.get_magnetic_declination()
                self._meas_settings.append(DocItem(_("Magn. Decl."), magn_decl, "°"))
            elif north_ref == BearingNorthRefs.REFERENCE_MARK_DIRECTION:
                ref__mark_dir = signalshark.scpi.bearing.get_reference_mark_direction()
                self._meas_settings.append(DocItem(_("Ref. Mark Dir."), ref__mark_dir, "°"))

            azimuth_corr = signalshark.scpi.bearing.get_azimuth_correction()
            self._meas_settings.append(DocItem(_("Azimuth Corr"), azimuth_corr, "°"))

            df_squelch_enabled = signalshark.scpi.bearing.get_dfsquelch_enable()
            if df_squelch_enabled:
                df_squelch = signalshark.scpi.bearing.get_dfsquelch()
                self._meas_settings.append(DocItem(_("DF Squelch"), df_squelch, unit))

            min_stability_enabled = signalshark.scpi.bearing.get_min_stability_enable()
            if min_stability_enabled:
                min_stability = signalshark.scpi.bearing.get_min_stability()
                self._meas_settings.append(DocItem(_("min. Stability"), min_stability, "dB"))

            min_dfquality_enabled = signalshark.scpi.bearing.get_min_dfquality_enable()
            if min_dfquality_enabled:
                min_dfquality = signalshark.scpi.bearing.get_min_dfquality()
                self._meas_settings.append(DocItem(_("min. DF Quality"), min_dfquality, "%"))

            ftune = (signalshark.scpi.bearing.get_frequency_tune())
            self._meas_settings.append(DocItem(_("Ftune"), ftune, "Hz", True))

            cbw = (signalshark.scpi.bearing.get_cbw())
            self._meas_settings.append(DocItem(_("CBW"), cbw, "Hz", True))

            meastime = signalshark.scpi.bearing.get_measurement_time()
            self._meas_settings.append(DocItem(_("Meas. Time"), meastime, "s", True))

            postavg = signalshark.scpi.bearing.get_post_avg()
            self._meas_settings.append(DocItem(_("Post Avg."), postavg, "s", True))

            cycle_time = signalshark.scpi.bearing.get_cycle_time()
            self._meas_settings.append(DocItem(_("Cycle Time"), cycle_time, "s", True))

            scan_time = signalshark.scpi.bearing.get_scan_time()
            self._meas_settings.append(DocItem(_("Scan Time"), scan_time, "s", True))

            scan_no = signalshark.scpi.bearing.get_scan_number()
            self._meas_settings.append(DocItem(_("Scan No."), scan_no))

            scan_count = signalshark.scpi.bearing.get_scan_count()
            self._meas_settings.append(DocItem(_("Scan Count"), scan_count))

            return self._take_screenshot(signalshark)
        except:
            return False


class DocMapDataItem(IDocDataItem):
    """Class that represents a Map View"""

    def __init__(self, screenshot_base_path: Optional[Path] = None, view_index: int = -1):
        """Class initialization"""
        super().__init__(screenshot_base_path, view_index)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return str(ViewTypes.MAP)

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("Map View")

    @property
    def screenshot_name(self) -> str:
        """Gets the screenshot name for this data item. """
        return _("Map_View")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 9

    @property
    def meas_result_headline(self) -> str:
        """Gets the measurement result headline for this data item. """
        return _("Localization Results")

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            # Read settings:
            # -------------------------------------------------------------------------------
            self._meas_settings.clear()
            map_name = signalshark.scpi.display.get_map_select()
            self._meas_settings.append(DocItem(_("Map Name"), map_name))

            map_center_lat, map_center_lon = signalshark.scpi.display.get_map_center_position()
            map_center_pos_str = "({}, {})".format(map_center_lat, map_center_lon)
            self._meas_settings.append(DocItem(_("Map Center Position"), map_center_pos_str))

            map_zoom_lev = signalshark.scpi.display.get_map_zoom_level()
            self._meas_settings.append(DocItem(_("Map Zoom Level"), map_zoom_lev))

            map_mode = signalshark.scpi.map.get_mode()
            self._meas_settings.append(DocItem(_("Map Mode"), map_mode))

            loc_area = signalshark.scpi.map.get_localization_area()
            loc_area_str = "({}, {}), ({}, {})".format(loc_area.top_left.latitude, loc_area.top_left.latitude,
                                                       loc_area.bottom_right.latitude, loc_area.bottom_right.latitude)
            self._meas_settings.append(DocItem(_("Localization Area"), loc_area_str))

            resolution = signalshark.scpi.map.get_localization_resolution()
            self._meas_settings.append(DocItem(_("Resolution"), resolution))

            bearing_error = signalshark.scpi.map.get_localization_bearing_error()
            self._meas_settings.append(DocItem(_("Bearing Error"), bearing_error, "°"))

            los_prob = signalshark.scpi.map.get_localization_los_probability()
            self._meas_settings.append(DocItem(_("LOS Prob."), los_prob, "%"))

            vel_squelch_enabled = signalshark.scpi.map.get_localization_min_speed_enable()
            if vel_squelch_enabled:
                vel_squelch = signalshark.scpi.map.get_localization_min_speed()
                self._meas_settings.append(DocItem(_("Velocity Squelch"), vel_squelch, "km/h"))

            min_dfqual_enabled = signalshark.scpi.map.get_localization_min_dfquality_enable()
            if min_dfqual_enabled:
                min_dfqual = signalshark.scpi.map.get_localization_min_dfquality()
                self._meas_settings.append(DocItem(_("min. DF Quality"), min_dfqual, "%"))

            loc_rec_select = signalshark.scpi.map.get_localization_record_select()
            self._meas_settings.append(DocItem(_("Selected Record"), loc_rec_select))

            # Read results:
            # -------------------------------------------------------------------------------
            self._meas_results.clear()

            # ToDo: Change measurement result according to map_mode
            if map_mode == MapModes.CONTINUES_LOCALIZATION or map_mode == MapModes.DISCRETE_LOCALIZATION:
                loc_data = signalshark.scpi.map.get_localization_data()
 
                self._meas_results.append(DocItem(_("Transmitter Frequency"), loc_data.frequency, "Hz", True))
                self._meas_results.append(DocItem(_("Bearing Count"), loc_data.bearing_count))
                self._meas_results.append(DocItem(_("Validity"), loc_data.valid))
                #self._meas_results.append(DocItem(_("Bearing Count"), loc_data.bearing_count))

                tx_pos = "({}, {})".format(loc_data.coordinate.latitude, loc_data.coordinate.longitude)
                self._meas_results.append(DocItem(_("Evaluated Transmitter Position"), tx_pos))

                loc_maj_semaxis = loc_data.semi_major_axis
                loc_min_semaxis = loc_data.semi_minor_axis
                loc_angle = loc_data.angle
                err_ellipse = "({}, {}, {})".format(loc_maj_semaxis, loc_min_semaxis, loc_angle)
                self._meas_results.append(DocItem(_("Error ellipse"), err_ellipse))

            # # Todo: Confirm result specs
            # if map_mode == MapModes.LOCALIZATION_RESULTS:
            #     loc_data = signalshark.scpi.map.get_localization_data()
            #
            # if map_mode == MapModes.MEASUREMENT_SITES:
            #     loc_data = signalshark.scpi.map.get_localization_data()
            #
            # if map_mode == MapModes.COVERAGE:
            #     loc_data = signalshark.scpi.map.get_localization_data()

            return self._take_screenshot(signalshark)
        except:
            return False


class DocDeviceDataItem(IDocDataItem):
    """Class that represents SignalShark basic information"""

    def __init__(self, screenshot_base_path: Optional[Path] = None):
        """Class initialization"""
        super().__init__(screenshot_base_path)

    @staticmethod
    def get_data_item_type() -> str:
        """Gets the data item type represented by this class"""
        return "DEVICE_DATA"

    @staticmethod
    def get_displ_name() -> str:
        """Gets the display name of the data item represented by this class"""
        return _("SignalShark Device Information")

    @property
    def settings_headline(self) -> str:
        """Gets the settings headline for this data item. """
        return _("Device Info")

    @property
    def sort_id(self) -> int:
        """Gets an ID to sort the data classes. """
        return 10

    def read_data_from_device(self, signalshark: SignalSharkDev) -> bool:
        """Reads out the measurement settings and data for this data item from the device.

        :param signalshark: The SignalShark device
        :return: Returns True if succeeded.
        """
        try:
            self._meas_settings.clear()

            part_num = signalshark.scpi.device.get_part_number()
            self._meas_settings.append(DocItem(_("P/N"), part_num))

            serial_num = signalshark.scpi.device.get_serial_number()
            self._meas_settings.append(DocItem(_("S/N"), serial_num))

            firmware = signalshark.scpi.device.get_application_version()
            self._meas_settings.append(DocItem(_("Firmware Version"), firmware))
            return True
        except:
            return False