#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import sys
import inspect
import os
from pathlib import Path
import wx
import time
import datetime as dt
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import MeasDlgThread
from nardascripting.base.toolbox import ListMsgBox
import nardascripting.base.toolbox as tb
from .gui.mdocdlgs import *
from .gui.mdocdlgsbase import *
from .basedata import IDocDataItem, DocData
from . import docitemdata as didata
from . import docmethods as dm
from . import filehandling as fh
from docxtpl import DocxTemplate, InlineImage
import jinja2
from . import langhelper as lh
_ = lh.CURRENT_GETTEXT


class MDoc(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = _("nslmdoc")
        self._scr_title = _("Meas. Documentation  V1.0")
        self._scr_description = _("Saves a screenshot of all Views to a .docx file")
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 1
        self._nsl_executed_behavior = NSL_Executed_Behaviors.SHOW_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        self.gen_settings = {}
        self.doc_settings = {}

    def _run_script(self, args):
        """ Script main function

        This method is called when a user clicks on the corresponding script button.
        """
        # Connect SignalShark
        # -------------------------------------------------------------------------------
        if not self.signalshark.connect():
            self.MessageBoxModal(_("Can not connect to device!"),
                                 "MDoc", wx.OK | wx.ICON_INFORMATION)
            return

        # Hold measurement acquisition
        task_state = self.signalshark.scpi.task.get_state()
        if task_state == TaskStates.RUN:
            self.signalshark.scpi.sense.hold()

        # Query all active tasks
        task_list = self.signalshark.scpi.task.get_names()
        task_type, task_name = self.signalshark.scpi.task.get_selected()

        # Load settings
        # -------------------------------------------------------------------------------
        gen_settings = fh.load_from_json(fh.GEN_SETTINGS_FILE_NAME, True)
        if gen_settings is None:
            self.MessageBoxModal(_("Please run Meas. Documentation Settings first!"),
                                 "MDoc", wx.OK | wx.ICON_INFORMATION)
            return

        doc_settings = {}
        tctr_default_meas_name = gen_settings.get("tctr_default_meas_name", "meas")
        with DocSettingsDlg(self.main_gui, task_list, task_name, tctr_default_meas_name) as doc_settings_dlg:
            if self.ShowDlgModalTop(doc_settings_dlg) == wx.ID_OK:
                doc_settings = doc_settings_dlg.doc_settings
            else:
                return

        self.signalshark.disconnect()

        # Create an additional measurement thread to keep the GUI alive during measurement.
        # -------------------------------------------------------------------------------
        # Please change 'MyUserScript._run_measurement' according to your class name.
        # With "args=[param1, parm2, paramn]" you can pass some optional parameters to the measurement thread.

        prg_dlg = MDocProgress(self.main_gui)
        mthread = MeasDlgThread(main_gui=self.main_gui, dlg=prg_dlg,
                                callback=MDoc._run_measurement,
                                args=[self.signalshark.addr, gen_settings, doc_settings])
        # Start the measurement thread and show a visualization (progress) dialog.
        mthread.start_measurement()

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         signalshark_addr: str, gen_settings: dict, doc_settings: dict):
        """ Main method of the measurement thread that handles a time consuming measurement.

        Please do NOT use any wxPython elements like a MessageBox or a Dialog here!
        They are only permitted in the GUI thread ('_run_script' method).
        Please use the "update_progress" method to display information!

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param signalshark_addr: Custom parameter to handover the ip address of the SignalShark
        :param gen_settings: General settings
        :param doc_settings: Settings data for document creation
        :return:
        """
        signalshark = SignalSharkDev(signalshark_addr)
        try:
            # Setup document data object
            doc_data = DocData()

            # Get/check doc settings
            # ---------------------------------------------------------------------------
            if doc_settings is None:
                return wait_for_stopevent(_("Unexpected issue. Please rerun script."))

            doc_template_filename = gen_settings.get("fpck_templ_file")
            if not doc_template_filename or not Path(doc_template_filename).exists():
                return wait_for_stopevent(_("Please define a valid Word template in the general settings dialog!"))

            base_working_dir_str = doc_settings.get("base_working_dir", "")
            if not base_working_dir_str:
                return wait_for_stopevent(_("Please define a valid base working directory!"))
            base_working_dir = Path(base_working_dir_str)
            if not base_working_dir.exists():
                return wait_for_stopevent(_("Please define a valid base working directory!"))
            if not "Datalogger" in base_working_dir_str:
                return wait_for_stopevent(_("Please define a valid base working directory!"))

            meas_name = doc_settings.get("meas_name", "")
            if not meas_name:
                return wait_for_stopevent(_("Please define a valid measurement directory name!"))

            selected_task = doc_settings.get("selected_task", "")
            if not selected_task:
                return wait_for_stopevent(_("Please select a measurement task first!"))

            doc_data.meas_name = meas_name
            posix_timestamp = doc_settings.get("posix_timestamp", 0)
            doc_data.time_stamp = dt.datetime.fromtimestamp(posix_timestamp)
            doc_data.author_name = doc_settings.get("author_name", "")
            doc_data.comment = doc_settings.get("comment", "")

            # Setup connection to the SignalShark and display error message if connection fails.
            # ---------------------------------------------------------------------------
            if not signalshark.connect():
                return wait_for_stopevent(_("Cannot connect to the device!"))
            signalshark.scpi.check_error()
            if stopevent.is_set():
                return

            # Create and set new working directory and get view information
            # ---------------------------------------------------------------------------
            # New Folder Creation + Set Working Directory:
            new_working_dir = str(base_working_dir.joinpath(meas_name))
            real_path_str = signalshark.scpi.dlogger.get_directory_add(new_working_dir)
            if not real_path_str:
                return wait_for_stopevent(_("Cannot create the specified working directory!"))
            signalshark.scpi.dlogger.set_dloger_working_directory(real_path_str)
            working_dir = Path(real_path_str)

            # Hold the measurements
            # signalshark.scpi.hold()

            # Setup measurement storage files:
            doc_filename = meas_name + ".docx"
            doc_full_filename = str(working_dir.joinpath(doc_filename))

            # Path for screenshots folder in SignalShark
            screenshot_base_path = working_dir.joinpath("Screen Shots")

            # Pass selected task to SignalShark
            signalshark.scpi.task.set_select(selected_task)
            time.sleep(2.0)

            # Get Task Type
            selected_task_type = signalshark.scpi.task.get_selected()[0]

            # Get available IDocDataItem classes
            # ---------------------------------------------------------------------------
            # inspect.getmembers(sys.modules[__name__], inspect.isclass)
            doc_data_item_handlers = {}
            for item in inspect.getmembers(didata, inspect.isclass):
                name = item[0]
                dcls = item[1]
                if issubclass(dcls, IDocDataItem):
                    doc_data_item_handlers[dcls.get_data_item_type()] = dcls

            # Find and add data handlers
            # ---------------------------------------------------------------------------
            doc_data.data_items.append(didata.DocBaseDataItem(screenshot_base_path))
            doc_data.data_items.append(didata.DocDeviceDataItem())
            enabled_markers = signalshark.scpi.marker.get_spectrum_marker_list()
            if len(enabled_markers) > 0:
                doc_data.data_items.append(didata.DocMarkerDataItem())

            # Get a list of views in the selected task
            views = signalshark.scpi.view.get_list()
            for view, index in views:
                view_type_name = str(view)
                view_handler = doc_data_item_handlers.get(view_type_name)
                if view_handler is not None:
                    doc_data.data_items.append(view_handler(screenshot_base_path, index))

            # Read data from device and take screenshots
            # ---------------------------------------------------------------------------
            # Hide progress dialog for screenshots
            update_progress(data=False)
            time.sleep(0.5)

            for data_item in doc_data.data_items:
                data_item.read_data_from_device(signalshark)

            # Show progress dialog
            update_progress(data=True)

            signalshark.scpi.check_error()

            # Save Measurement Datasets
            valid_comment = tb.get_valid_filename(doc_data.comment)
            signalshark.scpi.dlogger.save_dataset(meas_name, valid_comment)

            signalshark.scpi.check_error()

            # Render and save document
            # ---------------------------------------------------------------------------
            doc_data.data_items.sort(key=lambda x: x.sort_id, reverse=False)
            dm.create_document(doc_full_filename, doc_template_filename, doc_data)

            valid_comment = doc_data.comment.replace("\n", "<w:br/>")
            context = {
                "meas_name": doc_data.meas_name,
                "author_name": doc_data.author_name,
                "comment": valid_comment,
                "meas_date": doc_data.time_stamp.strftime("%Y-%m-%d"),
                "meas_time": doc_data.time_stamp.strftime("%H:%M")
            }
            doc = DocxTemplate(doc_full_filename)
            doc.render(context)
            doc.save(doc_full_filename)

        except Exception as e:
            update_progress(data=True)
            return wait_for_stopevent(str(e))
        finally:
            # Close connection
            if signalshark is not None:
                signalshark.disconnect()

# -------------------------------------------------------------------------------
#  To make use of the methods below, the following packages must be installed
#  and imported
# # for TOC
# import win32com.client
# import inspect, os
# import pythoncom
# -------------------------------------------------------------------------------

# # Update Table of Contents of .docx file
# def update_toc(docx_file):
#     import pythoncom
#     pythoncom.CoInitialize()
#     wordfile = win32com.client.DispatchEx("Word.Application")
#     doc = wordfile.Documents.Open(docx_file)
#     doc.TablesOfContents(1).Update()
#     doc.Close(SaveChanges=True)
#     wordfile.Quit()
#
#
# # Save .docx in .pdf format
# def convert_to_pdf(filepath: str):
#     global word
#     try:
#         word = win32com.client.DispatchEx("Word.Application")
#         target_path = filepath.replace(".docx", r".pdf")
#         word_doc = word.Documents.Open(filepath)
#         word_doc.SaveAs(target_path, FileFormat=17)
#         word_doc.Close()
#     except Exception as e:
#         raise e
#     finally:
#         word.Quit()
